#!/usr/bin/python3
import subprocess
import sys
import time

import gi
from gi.repository import GLib, UDisks

UDI = "/org/freedesktop/UDisks2/block_devices/sdb"
ESP_GUID = 'C12A7328-F81F-11D2-BA4B-00A0C93EC93B'

def launch_format(data=None):
    udisksclient = UDisks.Client.new_sync()
    obj = udisksclient.get_object(UDI)
    block = obj.props.block

    print("Creating partition table")
    block.call_format(
            'gpt',
            arg_options=GLib.Variant('a{sv}', None),
            cancellable=None,
            callback=format_finished,
            user_data=udisksclient)

    return False # Don't repeat

def format_finished(source_object, result, udisksclient):
    print("...format: finished")
    try:
        source_object.call_format_finish(result)
    except GLib.GError as e:
        if ('GDBus.Error:org.freedesktop.UDisks2.Error.Failed' in e.message and
                'Error synchronizing after initial wipe' in e.message):
            print("Failed to synchronize. Trying again, which usually solves the issue. Error was: %s" % e.message)
        obj = udisksclient.get_object(UDI)
        block = obj.props.block
        block.call_format(
            'gpt',
            arg_options=GLib.Variant('a{sv}', None),
            cancellable=None,
            callback=format_finished,
            user_data=udisksclient)
        return False

    print("Getting partition table")
    obj = udisksclient.get_object(UDI)
    partition_table = obj.props.partition_table
    print("Creating partition")
    partition_table.call_create_partition(
            arg_offset=0,
            arg_size=2500 * 2**20,
            arg_type=ESP_GUID,
            arg_name="Test",
            arg_options=GLib.Variant('a{sv}', None),
            cancellable=None,
            callback=create_partition_finished,
            user_data=udisksclient)

def create_partition_finished(source_object, result, udisksclient):
    print("...create partition: finished")
    try:
        source_object.call_create_partition_finish(result)
    except GLib.Error as e:
        # XXX: as of Debian Jessie, we often get errors while wiping
        # (Debian bug #767457). We ignore them as they are not fatal
        # for us... but we need to fix a few things later...
        if ('GDBus.Error:org.freedesktop.UDisks2.Error.Failed' in e.message and
             'Error wiping newly created partition' in e.message):
            print("Ignoring error %s" % e.message)
        else:
            raise
    print("Rescanning device")
    obj = udisksclient.get_object(UDI)
    block = obj.props.block
    block.call_rescan(GLib.Variant('a{sv}', None),
            None,
            callback=rescan_finished,
            user_data=udisksclient)

def rescan_finished(source_object, result, udisksclient):
    print("...rescan: finished")
    source_object.call_rescan_finish(result)

    print("Recreating object and reading partition table")
    obj = udisksclient.get_object(UDI)
    partition_table = obj.props.partition_table
    partitions = udisksclient.get_partitions(partition_table)
    print("partitions: %(partitions)s" % {'partitions': partitions})
    system_partition = partitions[0]

    print("Setting partition type")
    system_partition.call_set_type(ESP_GUID,
            GLib.Variant('a{sv}', None),
            callback=set_type_finished,
            user_data=udisksclient)

def set_type_finished(source_object, result, udisksclient):
    print("...set type: finished")
    source_object.call_set_type_finish(result)

    print("Recreating object and reading partition table")
    obj = udisksclient.get_object(UDI)
    partition_table = obj.props.partition_table
    partitions = udisksclient.get_partitions(partition_table)
    print("partitions: %(partitions)s" % {'partitions': partitions})
    system_partition = partitions[0]

    print("Setting partition name")
    system_partition.call_set_name("Test",
            GLib.Variant('a{sv}', None),
            callback=set_name_finished,
            user_data=udisksclient)

def set_name_finished(source_object, result, udisksclient):
    print("...set name: finished")
    source_object.call_set_name_finish(result)
    print("Done! You can now kill me with CTRL+C.")

main_loop = GLib.MainLoop()
GLib.idle_add(launch_format, None)
main_loop.run()
